/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#ifdef USE_NETWORK

#ifndef CLASS_NETWORK_SERVER_HEADER
#define CLASS_NETWORK_SERVER_HEADER

#include "../os/os.h"

namespace Network {

  class Connection;

  /**
   ** base class for a network server
   **
   ** @author	Diether Knof
   **/
  class Server : public OS {
    friend class Connection;
    public:
    static unsigned const STANDARD_PORT;

    public:
    static Server* new_();

    protected:
    // standard constructor
    Server();
    public:
    // destructor
    virtual ~Server();

    // the connections
    vector<Connection*>& connections();
    // the names of the addresses of this server
    virtual vector<string> local_addresses() const = 0;

    // start the server
    virtual void start();
    // stop the server
    virtual void stop();

    // sets the port
    void set_port(unsigned const port);

    // creates a new connection
    bool create_connection(string const& address, unsigned const port);


    // sends the text to all clients
    void send_to_all(string const& text);


    // 'player' has chatted 'message'
    void chat(string const& player, string const& message,
		    bool const self = true);

    // the settings have changed (here only the name is important)
    void setting_update(int const type, void const* const old_value);

    protected:
    // the connection is created
    void new_connection(Connection* const connection);
    // the connection is deleted
    void delete_connection(Connection* const connection);

    private:
    // port
    PRIV_VAR_R(unsigned, port);
    // whether the server is running
    PRIV_VAR_R(bool, is_running);

    // the connections to the server
    PRIV_VAR_R(vector<Connection*>, connections);

    private: // unused
    Server(Server const&);
    Server& operator=(Server const&);
  }; // class Server : public OS

} // namespace Network

extern Network::Server* server;

#endif // #ifndef CLASS_NETWORK_SERVER_HEADER

#endif // #ifdef USE_NETWORK
