/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM
#ifdef USE_NETWORK

#ifndef GTKMM_NETWORK_HEADER
#define GTKMM_NETWORK_HEADER

#include "base.h"
#include "widgets/sticky_dialog.h"
#include <gtkmm/liststore.h>
namespace Gtk {
  class TreeView;
  class CellRenderer;
  class StockButton;
  class CheckButton;
  class LabelSpinButton;
  class LabelEntry;
  class Container;
} // namespace Gtk

namespace Network {
  class Connection;
}

namespace UI_GTKMM_NS {
  /**
   ** @brief	the network dialog
   **
   ** @author	Diether Knof
   **/
  class Network : public Base, public Gtk::StickyDialog {
    class ConnectionModel : public Gtk::TreeModel::ColumnRecord {
      public:
	ConnectionModel();

	Gtk::TreeModelColumn<Glib::ustring> name;
	Gtk::TreeModelColumn<Glib::ustring> address;
	Gtk::TreeModelColumn<unsigned> port;
	Gtk::TreeModelColumn<Glib::ustring> found;
	Gtk::TreeModelColumn<Glib::ustring> color;
	Gtk::TreeModelColumn<bool> accept;
    }; // class ConnectionModel : public Gtk::TreeModel::ColumnRecord
    class LocalAddressesModel : public Gtk::TreeModel::ColumnRecord {
      public:
	LocalAddressesModel();

	Gtk::TreeModelColumn<Glib::ustring> name;
    };

    public:
      Network(Base* const parent);

      ~Network();

      // the server ist started
      void server_started();
      // the server ist stopped
      void server_stopped();
      // a new connection was established
      void network_connection_new(::Network::Connection const&
				  connection);
      // a connection has changed
      void network_connection_changed(::Network::Connection const&
				      connection);
      // a connection is deleted
      void network_connection_deleted(::Network::Connection const&
				      connection);
    private:
      void init();
      void update_sensitivity();
      void recreate_connections_list();
      void recreate_local_addresses_list();

      void server_running_change_event();
      void port_change_event();
      void reset_port_event();

      void new_connection_address_change_event();
      void connect_event();

    public:
      void language_update();

    private:
      Gtk::LabelSpinButton* port;
      Gtk::StockButton* standard_port;

      Gtk::CheckButton* server_running;

      ConnectionModel* connection_model;
      Glib::RefPtr<Gtk::ListStore> connection_list;
      Gtk::TreeView* connection_treeview;

      Gtk::Container* new_connection_container;
      Gtk::LabelEntry* new_connection_address;
      Gtk::LabelSpinButton* new_connection_port;
      Gtk::StockButton* connect;

      LocalAddressesModel* local_addresses_model;
      Glib::RefPtr<Gtk::ListStore> local_addresses_list;
      Gtk::TreeView* local_addresses_treeview;

    private: // unused
      Network();
      Network(Network const&);
      Network& operator=(Network const&);
  }; // class Network : public Base, public Gtk::StickyDialog

} // namespace UI_GTKMM_NS

#endif // #ifdef GTKMM_NETWORK_HEADER

#endif // #ifdef USE_NETWORK
#endif // #ifdef USE_UI_GTKMM
